#!/bin/bash
##########################
# CA Menu (c) Piotr Pisz #
#      2010.02.18        #
##########################
#Zmiany:
#2010.07.02 - dopisanie odnawiania certyfikatow.
#2014.04.02 - dopisanie SUBCA
#

PASS=AB1qaz@WSX!
DAYS=999
BAR="--------------------------------------------------------"
CADIR=/etc/pki/ca
OUTCNF=${CADIR}/cnf
OUTKEY=${CADIR}/key
OUTREQ=${CADIR}/req
OUTCRT=${CADIR}/crt
OUTP12=${CADIR}/p12
OUTCAK=${CADIR}/cak
OUTNEW=${CADIR}/new
OUTCRL=${CADIR}/crl
OUTPAK=${CADIR}/pak
OUTTXT=${CADIR}/txt
OUTTMP=${CADIR}/tmp
OUTVMW=${CADIR}/vmw
CNF=openssl.cnf
SUBJECT="/C=PL/ST=Mazowieckie/L=Warszawa/O=PWPW/OU=PU/CN="
SUBJECT_VM="/C=PL/ST=Mazowieckie/L=Warszawa/O=PWPW/OU="
PUSTE="nic"

bar_subj()
{ 
echo " "
echo -n " Enter ${SUBJECT}"
 }

bar_end()
{ 
echo " "
echo " Done!"
echo " "
 }

gen_full() 
{ 
    openssl req -config ${OUTCNF}/${CNF} -new -subj ${SUBJECT}${1}${4} -keyout ${OUTKEY}/${1}.key -out ${OUTREQ}/${1}.req -days ${DAYS} ${5}
    openssl ca -config ${OUTCNF}/${CNF} -policy ${2} -extensions ${3} -in ${OUTREQ}/${1}.req -out ${OUTCRT}/${1}.crt -passin pass:${PASS} -notext -batch
    openssl pkcs12 -export -in ${OUTCRT}/${1}.crt -inkey ${OUTKEY}/${1}.key -certfile ${OUTCAK}/${CHAIN} -out ${OUTP12}/${1}.p12 -passout pass:${PASS}
 }

gen_vm()
{ 
    openssl req -config ${OUTVMW}/openssl_vm.cnf -new -subj "${SUBJECT}${1}" -keyout ${OUTKEY}/${1}.key -out ${OUTREQ}/${1}.req -days ${DAYS} -nodes
    openssl ca -config ${OUTVMW}/openssl_vm.cnf -policy ${2} -in ${OUTREQ}/${1}.req -out ${OUTCRT}/${1}.crt -passin pass:${PASS} -notext -batch
    openssl pkcs12 -export -in ${OUTCRT}/${1}.crt -inkey ${OUTKEY}/${1}.key -certfile ${OUTCAK}/${CHAIN} -out ${OUTP12}/${1}.pkcs -passout pass:${PASS}
 }

issue_new()
{
    openssl ca -config ${OUTCNF}/${CNF} -policy ${2} -extensions ${3} -in ${OUTREQ}/${1} -out ${OUTCRT}/${1}.crt -passin pass:${PASS} -notext -batch
 }

sub_ca()
{ 
    SUBCA=`cat ${OUTCAK}/subca.txt`
    if [ ${SUBCA} -gt 0 ]; then
	echo " Intermediate CA ${CADIR}:"
	export CHAIN=ca.chain
    else
	echo " Root CA ${CADIR}:"
	export CHAIN=ca.crt
    fi
 }

create_cnf()
{ 
    cp ${OUTCNF}/openssl_vm.cnf ${OUTVMW}
    SNAME=`echo ${2} | awk -F'.' '{printf($1);}'`
    echo "subjectAltName			= DNS:${SNAME}, IP:${1}, DNS:${1}, DNS:${2}" >> ${OUTVMW}/openssl_vm.cnf
 }

create_esxi()
{ 
    cp ${OUTCNF}/openssl_vm.cnf ${OUTVMW}
    SNAME=`echo ${1} | awk -F'.' '{printf($1);}'`
    echo "subjectAltName			= DNS:${SNAME}, DNS:${1}" >> ${OUTVMW}/openssl_vm.cnf
 }

create_crt()
{ 
    create_cnf ${1} ${2}
    gen_vm ${2} policy_vm
    mkdir "${OUTVMW}/${3}"
    cp "${OUTCRT}/${2}.crt" "${OUTVMW}/${3}/rui.crt"
    cat ${OUTCAK}/${CHAIN} >> "${OUTVMW}/${3}/rui.crt"
    cp "${OUTKEY}/${2}.key" "${OUTVMW}/${3}/rui.key"
 }


vmware_crt()
{ 
while true
do
    clear
    echo ${BAR}
    echo " Generate a key / certificate for VMware"
    echo ${BAR}
    echo " 1 - Universal VMware server"
    echo " 2 - Prepare SSL for ESXi server"
    echo " 3 - Prepare SSL for vSphere environment"
    echo ${BAR}
    echo " 4 - Install certificates on ESXi (Require vSphere CLI!)"
    echo " 5 - Install certificates on vCSA (Must be run on vCSA!)"
    echo ${BAR}
    echo -n " Select (0 - Exit):"
    read VMI

    case ${VMI} in
	1)
	    echo " "
	    echo -n " Enter IP address:"
	    read IP
	    echo -n " Enter DNS hostname:"
	    read DNS
	    create_cnf ${IP} ${DNS}
	    gen_vm ${DNS} policy_vm
	    bar_end
	    read nic
	;;
	2)
	    echo " "
	    echo -n " Enter DNS hostname:"
	    read DNS
	    create_esxi ${DNS}
	    gen_vm ${DNS} policy_vm
	    bar_end
	    read nic
	;;
	3)
	    rm -r ${OUTVMW}/*
	    echo -n " Enter vCenter server IP:"
	    read IP
	    echo -n " Enter vCeter DNS hostname:"
	    read DNS
	    
	    OU="vCenter Web Client Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware Inventory Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware LDAP Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware Logbrowser Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware vCenter Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware vCenter VAMI Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    OU="VMware vSphere Autodeploy Service Certificate"
	    export SUBJECT="${SUBJECT_VM}${OU}/CN="
		create_crt ${IP} ${DNS} "${OU}"
	    
	    echo "Output on paths:"
	    ls -l ${OUTVMW}
	    bar_end
	    read nic
	;;
	4)
	    echo " "
	    echo " This procedure require vSphere CLI command: vifs"
	    echo -n " Enter DNS hostname:"
	    read DNS
	    echo -n " Enter password for root:"
	    read PASS
	    echo " "
	    cp ${OUTKEY}/${DNS}.key ${OUTVMW}/rui.key
	    cp ${OUTCRT}/${DNS}.crt ${OUTVMW}/rui.crt
	    vifs --server ${DNS} --username root --password ${PASS} --put ${OUTVMW}/rui.key /host/ssl_key
	    vifs --server ${DNS} --username root --password ${PASS} --put ${OUTVMW}/rui.crt /host/ssl_cert
	    
	    bar_end
	    read nic
	;;
	5)
	    echo " "
	    echo " The installation procedure takes a few minutes, press sometching to continue."
	    echo " "
	    read nic
	    ${CADIR}/bin/vcsa.sh
	    bar_end
	    read nic
	;;
	0)
	    break
	;;
	*)
	    echo " Bad choice ${VMI}, press Enter."
	    read nic
	;;
    esac
done
 }

menu_ca()
{ 
while true 
do
    clear
    echo ${BAR}
    echo " Initiating a new CA on path ${CADIR}"
    echo ${BAR}
    echo " 1 - Clearing ${CADIR} and initiating indexes."
    echo " 2 - Initiate Root CA."
    echo " 3 - Initiate Intermediate CA."
    echo ${BAR}
    echo " Handling of CA"
    echo ${BAR}
    echo " 4 - Generate new CRL list."
    echo " 5 - Revoke certificate."
    echo " 6 - New certification job."
    echo " 7 - Handling certification requests."
    echo ${BAR}
    echo " 8 - List expired certificate."
    echo " 9 - List acting certificates."
    echo "10 - Re-cert all expired certificates."
    echo ${BAR}
    echo -n " Select (0 - Exit):"
    read ICA
    
    case ${ICA} in

	1)
	    rm ${OUTKEY}/*.key
	    rm ${OUTREQ}/*.req
	    rm ${OUTCRT}/*.crt
	    rm ${OUTP12}/*.p12
	    rm ${OUTP12}/*.pkcs
	    rm ${OUTCAK}/ca.*
	    rm ${OUTNEW}/*.pem
	    rm ${OUTPAK}/pa*
	    rm ${OUTTXT}/*.txt
	    rm ${OUTCRL}/*.pem
	    rm ${OUTCRL}/*.der
	    rm -r ${OUTVMW}/*
	    echo "01" > ${OUTCNF}/serial
	    echo "00" > ${OUTCNF}/crlnumber
	    rm ${OUTCNF}/index.txt
	    touch ${OUTCNF}/index.txt
	    
	    bar_end
	    read nic
	;;
	2)
	    openssl req -config ${OUTCNF}/${CNF} -new -x509 -days ${DAYS} -keyout ${OUTCAK}/ca.key -out ${OUTCAK}/ca.crt -passin pass:${PASS}
	    openssl x509 -in ${OUTCAK}/ca.crt -out ${OUTCAK}/ca.txt -text
	    echo "0" > ${OUTCAK}/subca.txt
	    bar_end
	    read nic
	;;
	3)
	    openssl genrsa -aes256 -out ${OUTCAK}/ca.key -passout pass:${PASS} 4096
	    read
	    openssl req -config ${OUTCNF}/${CNF} -new -key ${OUTCAK}/ca.key -out ${OUTREQ}/ca.req -passin pass:${PASS}
	    echo " "
	    echo " Send ${OUTREQ}/ca.req file to Root CA."
	    echo " Place Intermediate CA certificate in file: ${OUTCAK}/ca.crt"
	    echo " Place Root CA certficate in file: ${OUTCAK}/ca.chain (as described in ${OUTCAK}/chain.readme)"
	    echo "1" > ${OUTCAK}/subca.txt
	    bar_end
	    read nic
	;;
	4)
	    openssl ca -config ${OUTCNF}/${CNF} -gencrl -out ${OUTCRL}/crl.pem -passin pass:${PASS} -batch
	    openssl crl -in ${OUTCRL}/crl.pem -outform der -out ${OUTCRL}/crl.der
	    
	    bar_end
	    read nic
	;;
	5)
	    echo " "
	    echo -n " Certificate ${OUTCRT}/"
	    read CERTYF
	    openssl ca -config ${OUTCNF}/${CNF} -revoke ${OUTCRT}/${CERTYF} -passin pass:${PASS} -batch
	    
	    bar_end
	    read nic
	;;
	6)    
	    echo -n " Enter key name ${OUTKEY}/"
	    read KEY
	    openssl req -config ${OUTCNF}/${CNF} -new -keyout ${OUTKEY}/${KEY}.key -out ${OUTREQ}/${KEY}.req -days ${DAYS}
	    
	    bar_end
	    read nic
	;;
	7)
	    echo -n " Enter request name ${OUTREQ}/"
	    read REQ
	    openssl ca -config ${OUTCNF}/${CNF} -policy policy_server -extensions srv_crt -out ${OUTCRT}/${REQ}.crt -infiles ${OUTREQ}/${REQ}.req
	    
	    bar_end
	    read nic
	;;
	8)
	
	    openssl ca -config ${OUTCNF}/${CNF} -gencrl -out ${OUTCRL}/crl.pem -passin pass:${PASS} -batch
            openssl crl -in ${OUTCRL}/crl.pem -outform der -out ${OUTCRL}/crl.der
	    rm ${OUTTMP}/expire.txt
	    
	    cd ${OUTCRT}
	    CERTY=`ls -1 *.crt`
	    for CHECK in ${CERTY}
	    do
		CHCRT=`openssl verify -CAfile ${OUTCAK}/ca.crt ${OUTCRT}/${CHECK} | grep expire | wc -c`
		#echo ${CHCRT}
		if [ ${CHCRT} -gt 10 ]; then
		    echo "Wygas: ${CHECK}"
		    echo ${CHECK} | awk -F'.' '{printf($1);}' >> ${OUTTMP}/expire.txt
		    echo "" >> ${OUTTMP}/expire.txt
		fi
	    done
	
	    bar_end
	    read nic
	;;
	9)
	    rm ${OUTTMP}/runcrt.txt
	    
	    cd ${OUTCRT}
	    CERTY=`ls -1 *.crt`
	    for CHECK in ${CERTY}
	    do
		CHCRT=`openssl verify -CAfile ${OUTCAK}/ca.crt ${OUTCRT}/${CHECK} | grep expire | wc -c`
		#openssl verify -CAfile ${OUTCAK}/ca.crt ${OUTCRT}/${CHECK}
		#echo ${CHCRT}		
		if [ ${CHCRT} == 0 ]; then
		    echo "Wazny: ${CHECK}"
		    echo ${CHECK} | awk -F'.' '{printf($1);}' >> ${OUTTMP}/runcrt.txt
		    echo "" >> ${OUTTMP}/runcrt.txt
		fi    
	    done
	
	    bar_end
	    read nic
	;;
	10)
	    echo " "
	    echo " Please make first 8 from this menu."
	    echo " We use file ${OUTTMP}/expire.txt"
	    echo " "
	    echo " Please press sometching, you make mass recertification"
	    echo " for all expired certificates!"
	    read nic
	    cp -f ${OUTTMP}/expire.txt ${OUTTXT}/lista.txt
	    rm ${OUTPAK}/*.tar
	    rm ${OUTPAK}/*.txt
	    ${CADIR}/bin/masscert.sh
	    
	    bar_end
	    read nic
	;;
	0)
	    break
	;;
	*) 
	    echo " Bad choice. Press Enter."
	    read nic
	;;
    esac
done
 }

menu_crt()
{ 
while true 
do
    clear
    echo ${BAR}
    echo " Handling of certificates and requests"
    echo ${BAR}
    echo " 1 - Generate key pair for User."
    echo " 2 - Generate key pair for incoming server."
    echo " 3 - Generate key pair for outcoming server."
    echo " 4 - SAN certificate from config file."
    echo " 5 - SAN certificate (enter mannually hostnames)."
    echo " 6 - Sign certificate request."
    echo ${BAR}
    echo " 7 - Service tasks for VMware."
    echo ${BAR}
    echo -n " Enter (0 - Exit):"
    read CRT

    case ${CRT} in
	1)
	    bar_subj
		read NAME
		echo -n " Enter ${SUBJECT}${NAME}/emailAddress="
		read EMAIL
		gen_full ${NAME} policy_user usr_crt "/emailAddress=${EMAIL}"
	    bar_end
	    read nic
	;;
	2)
	    bar_subj
		read NAME
		gen_full ${NAME} policy_server srv_crt "/" -nodes
	    bar_end
	    read nic
	;;
	3)
	    bar_subj
		read NAME
		gen_full ${NAME} policy_server srvcli_crt "/" -nodes
	    bar_end
	    read nic
	;;
	4)
	    bar_subj
		read NAME
		gen_full ${NAME} policy_server multisrv_crt "/" -nodes
	    bar_end
	    read nic
	;;
	5)
	    bar_subj
		read NAME
		echo " "
		echo " Please enter SAN domains as described:"
		echo " DNS:host1.domain.pl,DNS:IP,DNS:host2.domena.pl etc."
		echo " "
		echo -n " Enter:"
		read DNS
		cp -f ${OUTTMP}/ext.src ${OUTTMP}/ext.tmp
		echo "subjectAltName			= ${DNS}" >> ${OUTTMP}/ext.tmp
		openssl genrsa -out ${OUTKEY}/${1}.key 1024
		openssl req -config ${OUTCNF}/${CNF} -new -subj ${SUBJECT}${NAME} -key ${OUTKEY}/${1}.key -out ${OUTREQ}/${1}.req
		openssl ca -config ${OUTCNF}/${CNF} -policy policy_server -extfile ${OUTTMP}/ext.tmp -in ${OUTREQ}/${1}.req -out ${OUTCRT}/${1}.crt -passin pass:${PASS} -batch		
	    bar_end
	    read nic
	;;
	6)
	       echo " "
	       echo -n " Enter request name: ${OUTREQ}/"
	       read NAME
	       issue_new ${NAME} policy_server srv_crt "/" -nodes
	   bar_end
	   read nic
	;;
	7)
	    vmware_crt
	;; 
	0)
	    break
	;;
	*) 
	    echo " Bad choice. Press Enter."
	    read nic
	;;
    esac 
done
 }

while true 
do
    clear
    echo ${BAR}
    sub_ca
    echo ${BAR}
    echo " 1 - Iniciate or service new CA."
    echo " 2 - Certificate, key or requests service."
    echo " 3 - Mass certification."
    echo ${BAR}
    echo -n " Enter (0 - Exit):"
    read KL

    case ${KL} in
	1) 
	    menu_ca
	;;
	2)
	    menu_crt
	;;
	3)
	    echo " "
	    echo " Put file with key names in ${CADIR}/txt/lista.txt and press enter"
	    echo " "
	    read nic
	    ${CADIR}/bin/masscert.sh
	    echo ${BAR}
	    echo " Done! Press enter!"
	    echo ${BAR}
	    read nic
	;;
	0)
	    exit
	;;
	*)
	    echo "Bad choice. Press Enter."
	    read nic
	;;
    esac
done
